/*
 * RoboDroid Wars
 * Copyright (C) 2006 Dawid Ciezarkiewicz <dawid.ciezarkiewicz@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA 
 */

/**
 * License: GNU GPL
 *
 * Author: Dawid Ciężarkiewicz <dawid.ciezarkiewicz@gmail.com>
 */
module battle;

import map;
import options;
import fx;
import player;
import droid;

import std.thread;
import std.c.linux.linux;

/**
 * Single battle.
 */
class Battle {
	private uint tickDuration;
	private ulong lastTickTime;
	private Options options;
	private Map map;
	private FX fx;
	private Droid droids[];

	/**
	 * Create battle.
	 */
	public this(char[] name, Options options, FX fx) {
		map = new Map(this, name);
		fx.registerBattle(this);
		this.options = options;
		this.fx = fx;

		uint speed = options.getSimulationSpeed();
		tickDuration = cast(int) ((6000000.0)
				/ ((speed * speed * speed * speed) + 1));
	}

	/**
	 * Delete battle.
	 */
	public ~this() {
		delete map;
	}

	/**
	 * Spawn players onto the battle map.
	 */
	public void joinPlayers(Player[] players) {
		foreach (Player player; players) {
			joinPlayer(player);
		}
	}

	/**
	 * Spawn player.
	 */
	public void joinPlayer(Player player) {
		droids ~= map.spawnDroid(player);
	}

	/**
	 * Run battle.
	 */
	public void run() {
		while (getAliveDroidCounter() > 1) {
			runTick();
		}
	}

	/**
	 * Run one single tick.
	 */
	private void runTick() {
		map.runTick();

		timeval current_time;
		gettimeofday(&current_time, null);
		ulong time = current_time.tv_sec * 1000 * 1000
			+ current_time.tv_usec;

		if (time < (lastTickTime + tickDuration)) {
			Thread.getThis().wait(
					cast(uint)((lastTickTime + tickDuration) - time)
					);
		}
		lastTickTime = time;

		fx.flush();
	}

	/**
	 * Get amount of alive droids.
	 */
	private uint getAliveDroidCounter() {
		uint alive = 0;
		foreach(Droid d; droids) {
			if (d.isAlive())
				alive++;
		}
		return alive;
	}

}
