/*
 * RoboDroid Wars
 * Copyright (C) 2006 Dawid Ciezarkiewicz <dawid.ciezarkiewicz@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 */

/**
 * License: GNU GPL
 *
 * Author: Dawid Ciężarkiewicz <dawid.ciezarkiewicz@gmail.com>
 */
module cell.cell;

import cell.autonomous;
import cell.movable;

import map;

/**
 * Each object on the map is separate cell.
 */
class Cell {
	public uint getX() {
		return x;
	}

	public uint getY() {
		return y;
	}

	protected:
		/**
		 * X position.
		 */
		uint x;
		
		/**
		 * Y position.
		 */
		uint y;

		/**
		 * Parent map.
		 */
		Map map;

		/**
		 * Constructor.
		 */
		this(Map map, uint x, uint y) {
			this.map = map;
			this.x = x;
			this.y = y;
		}


		/**
		 * Try to scan cell.
		 *
		 * Params:
		 * autonomous = scanning object
		 *
		 * Returns:
		 * Discovered = scan was successful and something was found.
		 * null       = scan returned nothing (cell not scannable) 
		 */
	   	abstract Discovered tryScan(Autonomous autonomous);

		/**
		 * Returned on successful scan.
		 */
		class Discovered {
			char[] objectId;
			public:
				this(char[] objectId) {
					this.objectId = objectId;
				}
		}

		/**
		 * Try to move onto the cell.
		 *
		 * Return:
		 * true  = move successful
		 * false = move failed
		 */
		bool tryStepIn(Movable m) {
			return false;
		}
		
		/**
		 * Try shooting into cell.
		 *
		 * Returns:
		 * true  = bullet was intercepted by cell
		 * false = bullet was not intercepted
		 */
		bool tryShoot(Autonomous a) {
			return true;
		}

		/**
		 * Is cell on the top of the others?
		 *
		 * Since one cell can be placed on the top of the other
		 * this is used to check if cell shouldn't be repainted
		 * or something like that.
		 */
		bool isOnTop() {
			return ((map.getCell(x, y)) is this);
		}

		/**
		 * Repaint cell on the screen.
		 */
		void repaint() {
		}

		/**
		 * Get bitmaps that should be used when repainting.
		 */
		abstract char[][] getBitmapNames();
}
