/*
 * RoboDroid Wars
 * Copyright (C) 2006 Dawid Ciezarkiewicz <dawid.ciezarkiewicz@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA 
 */

/**
 * License: GNU GPL
 *
 * Author: Dawid Ciężarkiewicz <dawid.ciezarkiewicz@gmail.com>
 */
module cell.scannable;

import cell.autonomous;
import map;
import cell.cell;

/**
 * Parent class for cells that wish to indicate that they were scanned.
 */
class Scannable : public Cell {
	private uint scanDisplayDuration;

	/**
	 * Constructor.
	 */
	public this(Map map, uint x, uint y) {
		super (map, x, y);
	}

	public void tick() {
		if (scanDisplayDuration > 0) {
			if (isOnTop()) {
				if (--scanDisplayDuration == 0) {
					repaint();
				}
			} else {
				scanDisplayDuration = 0;
			}
		}
	}

	public override Discovered tryScan(Autonomous a) {
		scanDisplayDuration = 15;
		repaint();
		return null;
	}

	public override char[][] getBitmapNames() {
		char[][] ret;
		if (scanDisplayDuration > 0) {
			ret ~= "scan";
		}
		return ret;
	}
}
