/*
 * RoboDroid Wars
 * Copyright (C) 2006 Dawid Ciezarkiewicz <dawid.ciezarkiewicz@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 */

/**
 * License: GNU GPL
 *
 * Author: Dawid Ciężarkiewicz <dawid.ciezarkiewicz@gmail.com>
 */
module fx;

private {
	import options;
	import battle;
	import io;
	import video;
	import audio;
	import fxsdl;
}

/**
 * Interface for implementations of I/O, Video and Audio subsystems.
 */
class FX {
	public:
		static FX Init(Options options) {
			return new SDLFX(options);
		}

		/**
		 * Register new battle.
		 *
		 * Should be called each time new battle begins to
		 * update necessary information.
		 */
		abstract void registerBattle(Battle b);

		/**
		 * Flush all subsystems.
		 *
		 * i.e. changes to 2d display should be visible simultaneously 
		 * at the end of each tick. This will do the thing.
		 */
		abstract void flush();
		
		/**
		 * Get video driver.
		 */
		abstract VideoDriver getVideo();

		/**
		 * Get audio driver.
		 */
		abstract AudioDriver getAudio();

		/**
		 * Get io driver.
		 */
		abstract IODriver getIO();
}
