/*
 * RoboDroid Wars
 * Copyright (C) 2006 Dawid Ciezarkiewicz <dawid.ciezarkiewicz@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 */

/**
 * License: GNU GPL
 *
 * Author: Dawid Ciężarkiewicz <dawid.ciezarkiewicz@gmail.com>
 */
module fxsdl;

import fx;

import audio;
import audionull;
import audiosdl;
import video;
import videosdl;
import videonull;
import io;
import ioconsole;
/* import derelict.sdl.sdl;
import derelict.sdl.mixer;
import derelict.util.exception;
*/
import battle;
import options;

class SDLFX : public FX {
	protected VideoDriver video;
	protected AudioDriver audio;
	protected IODriver io;

	public this(Options options) {
		if (isSDLAvailable()) {
			video = new SDLVideo(options);
		} else {
			video = new NullVideo();
		}

		if (isSDLMixerAvailable()) {
			audio = new SDLAudio(options);
		} else {
			audio = new NullAudio();
		}

		io = new ConsoleIO(options);

	}

	public static this() {
		/+try {
			DerelictSDL.load();
			SDLSuccessful = true;
		} catch (SharedLibLoadException except) {
			SDLSuccessful = false;
		}

		try {
			DerelictSDLMixer.load();
			SDLMixerSuccessful = true;
		} catch (SharedLibLoadException except) {
			SDLMixerSuccessful = false;
		}+/
	}

	private static bool SDLSuccessful;
	private static bool SDLMixerSuccessful;

	public bool isSDLAvailable() {
		return SDLSuccessful;
	}

	public bool isSDLMixerAvailable() {
		return SDLMixerSuccessful;
	}

	override void registerBattle(Battle b) {
	}

	override void flush() {
	}

	override VideoDriver getVideo() {
		return video;
	}

	override AudioDriver getAudio() {
		return audio;
	}

	override IODriver getIO() {
		return io;
	}
}


