/*
 * RoboDroid Wars
 * Copyright (C) 2006 Dawid Ciezarkiewicz <dawid.ciezarkiewicz@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA 
 */

/**
 * License: GNU GPL
 *
 * Author: Dawid Ciężarkiewicz <dawid.ciezarkiewicz@gmail.com>
 */
module rdwars;

import options;
import battle;
import player;
import fx;
import error;

/**
 * Main program class.
 *
 * It handles options, players and each battle.
 */
class RDWars {
	private Options options;
	private FX fx;

	/**
	 * Initialize simulation with provided command line options.
	 */
	public this(char[][] args) {
		options = new Options(args);
		fx = FX.Init(options);
	}

	/**
	 * Start simulation.
	 */
	public void run() {
		Player players[];

		foreach (char[] player; options.getPlayerNames()) {
			players ~= new Player(player);
		}

		try {
			foreach (char[] battle; options.getBattleNames()) {
				Battle b = new Battle(battle, options, fx);
				b.joinPlayers(players);
				b.run();
			}
		} catch (SimulationEndRequest e) {
		}
	}
}

/**
 * Main program entry.
 *
 * args - program command line options
 */
int main(char[][] args) {
	RDWars g;
	try {
		g = new RDWars(args);
	} catch (SimulationEndRequest e) {
	}
	g.run();
	delete g;
	return 0;
}

